<?php
namespace pjp\core\classmanagement;

/**
 * PJPClassLoader offers all the functionality needed for auto-loading classes on demand.
 * Use loadClassFromCache() to attempt finding the location of the class to load from a cache,
 * and loadClass() to find the location of the class in runtime. Don't forget to register
 * the loading functions in spl_autoload_register (see http://ca3.php.net/manual/en/function.spl-autoload-register.php)
 */
final class PJPClassLoader {
	private static $classcache = array();
	private static $observers = array();

	private static $usingcache = false;
	private static $cachedumpneeded = false;

//LOADING FUNCTIONS

	/**
	 * Attempts to load a class using the location stored in the class cache.
	 */
	public static function loadClassFromCache($class) {
		self::$usingcache = true;
		$classuri = self::getClassURIFromCache($class);
		
		if( $classuri && include_once($classuri) )
			self::notifyObservers($class);
	}

	/**
	 * Attempts to load a class, finding it's location in runtime through spl_autoload.
	 */
	public static function loadClass($class) {
		$classcount = count( get_included_files() );
		$classuri = self::forwardSlash($class);
		spl_autoload($classuri);

		if(self::$usingcache && count( get_included_files() ) > $classcount) {
			self::updateCache( $class, self::getLastIncludedFile() );
			self::notifyObservers($class);
		}
	}

	/**
	 * Fallback function for throwing an exception should the class loading fail.
	 */
	public static function classNotFound($class) {
		throw ClassException::loadingFailed($class);
	}

//OBSERVER FUNCTIONS

	/**
	 * Register callback functions to be notified when a class is loaded.
	 */
	public static function addObserver($observer) {
		self::$observers[$observer] = $observer;
	}

	/**
	 * Unregisters callback function from being notified when a class is loaded.
	 */
	public static function removeObserver($observer) {
		unset(self::$observers[$observer]);
	}

	/**
	 * Notify observers when a class is loaded.
	 * Returns a hash {'class', 'file'} with their respective values set.
	 */
	private static function notifyObservers($class) {
		if(count(self::$observers) > 0) {
			$included = self::getLastIncludedFile();
			foreach(self::$observers as $observer)
				call_user_func($observer, array('class'=>$class, 'file'=>$included) );
		}
	}

//UTIL FUNCTIONS

	//Helper method for finding the most recent file included.
	private static function getLastIncludedFile() {
		$files = get_included_files();
		$file = end($files);
		return $file;
	}

	//Helper method for converting path separators to /
	private static function forwardSlash($class) {
		return strtr( $class, array('\\' => '/') );
	}

//CLASS CACHE FUNCTIONS
	//Lookup the path of a class in the class cache. Prep the cache if not yet built.
	private static function getClassURIFromCache($class) {
		if(! self::$classcache)
			self::prepClassCache();

		return self::$classcache[$class];
	}

	//helper method for updating the class cache hash.
	private static function updateCache($class, $path) {
		self::$classcache[$class] = $path;
		self::$cachedumpneeded = true;
	}

	//Helper method for prepping the class cache. Reads the serialized class cache file.
	private static function prepClassCache() {
		$cachedump = @ file_get_contents(PJP_CLASS_CACHE_PATH);
		if($cachedump)
			self::$classcache = unserialize($cachedump);
	}

	//helper method for dumping a serialized class cache to file.
	public static function writeClassCache() {
		if(self::$usingcache && self::$cachedumpneeded) {
			$cachedump = serialize(self::$classcache);
			@ file_put_contents(PJP_CLASS_CACHE_PATH, $cachedump);
		}
	}

}

//Registers class cache to be serialized and dumped on script termination
register_shutdown_function(__NAMESPACE__ . '\PJPClassLoader::writeClassCache');

?>