<?php
namespace pjp\core\pluginmanagement;

use pjp\core\tools\PathBuilder;

/**
 * Implementation of AbstractPluginLoader that adds plugin directories to the include path.
 * The mapping of namespaces to directory structure allows this implementation to nicely
 * integrate with the classloader without any modification. PJPPluginLoader is a singleton.
 */
class PJPPluginLoader extends AbstractPluginLoader {
	private static $instance;

	private $pluginpaths;

	protected function __construct() {
		parent::__construct();

		$this->pluginpaths = array();
	}

	/**
	 * Static convenience method for loading plugins
	 */
	public static function load($pluginid) {
		self::getPluginLoader()->loadPlugin($pluginid);
	}

	/**
	 * Get the plugin loader
	 */
	public static function getPluginLoader() {
		if (! isset(self::$instance) )
			self::$instance = new self();

		return self::$instance;
	}

	//Is plugin already registered in the plugin library?
	protected function pluginIsLoaded($pluginid) {
		return PluginLibrary::getLibrary()->pluginIdIsRegistered($pluginid);
	}

	//Does the plugin physically exist?
	protected function pluginExists($pluginid) {
		$pluginpath = $this->lookupAndCachePluginPath($pluginid);
		return is_dir($pluginpath);
	}

	//Build the plugin
	protected function buildPlugin($pluginid) {
		$pluginbuilder = new PluginBuilder();
		$pluginpath = $this->lookupAndCachePluginPath($pluginid);

		$plugindescriptionfile = PathBuilder::buildPath($pluginpath, PJP_PLUGIN_MANIFEST);
		$pluginmanifest = file_get_contents($plugindescriptionfile);

		try {
			$pluginbuilder->processManifestFromString($pluginmanifest);
		}
		catch(Exception $e) {
			throw PluginLoaderException::invalidManifest($pluginid, $e);
		}

		return $pluginbuilder->getProduct();
	}

	//Load the plugin's dependancies.
	protected function loadDependantPlugins(Plugin $plugin) {
		foreach ($plugin->dependancies as $dependancy)
			$this->loadPlugin($dependancy);
	}

	//Register plugin in the plugin library.
	protected function registerPlugin(Plugin $plugin) {
		$this->addPluginToPath($plugin);
		PluginLibrary::getLibrary()->registerPlugin($plugin);
	}

	//Run plugin's activator.
	protected function activatePlugin(Plugin $plugin) {
		$pluginactivator = $plugin->activator;
		if ( empty($pluginactivator) )
			return;

		$activatorobject = new $pluginactivator();
		if (! ($activatorobject instanceof PluginActivator) )
			throw PluginLoaderException::invalidActivator($plugin->id);
		$activatorobject->activatePlugin();
	}

	//Add plugin location to include path.
	private function addPluginToPath($plugin) {
		$includepath = get_include_path();
		$includepath .= PATH_SEPARATOR.$this->lookupAndCachePluginPath($plugin->id);
		set_include_path($includepath);	
	}

	//elper method for finding the physical plugin in the platform or application directory.
	private function lookupAndCachePluginPath($pluginid) {
		if(! isset($this->pluginpaths[$pluginid]) ) {
			$platformpluginpath = PathBuilder::buildPlatformPath($pluginid);
			$applicationpluginpath = PathBuilder::buildApplicationPath($pluginid);

			if( is_dir($platformpluginpath) )
				$this->pluginpaths[$pluginid] = $platformpluginpath;
			elseif( is_dir($applicationpluginpath) )
				$this->pluginpaths[$pluginid] = $applicationpluginpath;
			else
				$this->pluginpaths[$pluginid] = '';
		}
		
		return $this->pluginpaths[$pluginid];
	}

}

?>